#include <iostream>
#include <vector>
#include <cmath>
#include "polinom.h"

using namespace std;

Polinom::Polinom()
{
    coefficients.push_back(0);
}

Polinom::Polinom(const vector<double> &coefficients)
{
    this->coefficients = coefficients;
    simplify();
}

double Polinom::getCoefficient(int index) const
{
    if(index >= 0 && index < coefficients.size())
        return coefficients[index];
    else
        return 0;
}

void Polinom::setCoefficient(int index, double value)
{
    if(index < 0)
        return;
    if(index >= coefficients.size())
        coefficients.resize(index + 1);
    coefficients[index] = value;

    if(value == 0)
        simplify();
}

double Polinom::evaluate(double x) const
{
    double result = 0;
    for(int i = 0; i < coefficients.size(); ++i)
        result += coefficients[i] * pow(x, i);
    return result;
}

void Polinom::simplify()
{
    int lastIndex = coefficients.size() - 1;
    while(lastIndex > 0 && coefficients[lastIndex] == 0)
        --lastIndex;
    if(lastIndex < coefficients.size() - 1)
        coefficients.resize(lastIndex + 1);
}

Polinom operator+(const Polinom &a, const Polinom &b)
{
    Polinom c = a;
    for(int i = 0; i < b.coefficients.size(); ++i)
        c.setCoefficient(i, c.getCoefficient(i) + b.coefficients[i]);
    return c;
}

Polinom operator-(const Polinom &a, const Polinom &b)
{
    Polinom c = a;
    for(int i = 0; i < b.coefficients.size(); ++i)
        c.setCoefficient(i, c.getCoefficient(i) - b.coefficients[i]);
    return c;
}

Polinom operator*(const Polinom &a, const Polinom &b)
{
    Polinom c;
    for(int i = 0; i < a.coefficients.size(); ++i)
        for(int j = 0; j < b.coefficients.size(); ++j)
            c.setCoefficient(i + j, c.getCoefficient(i + j) + a.coefficients[i] * b.coefficients[j]);
    c.simplify();
    return c;
}

std::istream& operator>>(std::istream &in, Polinom &p)
{
    int n;
    in >> n;
    if(n > 0)
    {
        p.coefficients.resize(n);
        for(int i = p.coefficients.size() - 1; i >= 0; --i)
            in >> p.coefficients[i];
    }
    else
    {
        p.coefficients.resize(1);
        p.coefficients[0] = 0;
    }
    return in;
}

std::ostream& operator<<(std::ostream &out, const Polinom &p)
{
    for(int i = p.coefficients.size() - 1; i >= 0; --i)
    {
        out << p.coefficients[i] << "x^" << i;
        if(i > 0)
            out << " + ";
    }
    return out;
}
